/***************************************************

KABOOM BOX by Cameron Coward (cameroncoward.com)

An RFID-controlled music device in a vintage
Panasonic "TNT" 8 track player.

Music stored on DFPlayer Mini SD card,
then folders selected based on scanned RFID

Works with MIFARE Classic 1K (ISO 14443A 13.56MHz) tags

-Raspberry Pi Pico Dev Board
-HiLetgo PN532 v3 RFID Scanner
-HiLetgo SSD1331 0.95" 65k SPI OLED Display
-DFPlayer Mini MP3 Player/Amplifier

 ****************************************************/

#include <SPI.h>                  // for display communication
#include <Wire.h>                 // for RFID communication
#include <DFRobotDFPlayerMini.h>  // for DFPlayer Mini
#include <Adafruit_GFX.h>         // for drawing graphics
#include <Adafruit_SSD1331.h>     // for HiLetgo SSD1331 display
#include <PN532_I2C.h>            // for PN532 RFID
#include <PN532.h>                // for PN532 RFID
#include <NfcAdapter.h>           // for PN532 RFID
#include "iconSet.h"              // custom icons and logo

// For display
#define sclk 15                     // SCL on HiLetgo display
#define mosi 14                     // SDA on HiLetgo display
#define cs   11                     // CS on HiLetgo display
#define rst  13                     // RES on HiLetgo display
#define dc   12                     // DC on HiLetgo display

#define BUTTON 2

// Color definitions
#define	BLACK           0x0000
#define	BLUE            0x001F
#define	RED             0xF800
#define	GREEN           0x07E0
#define CYAN            0x07FF
#define MAGENTA         0xF81F
#define YELLOW          0xFFE0
#define WHITE           0xFFFF
#define PINK            0xF810

String tagId = "None";              // this will be filled by each scanned tag's ID
String previousId = "";             // this will store the previously scanned tag's ID to prevent playing the same tag over and over

String tagList[3] = {               // a list of every tag ID we want to actually use (can add more)
  "3A 65 75 BD",
  "8A 31 75 BD",
  "7A D8 75 BD"
};

String nameList[3] = {              // a list of names for each playlist/genre/album (order must match tagList)
  "  LOVIN",                        // spacing is important for centering, as are the number of characters. 9 characters is good, spaces preceding to center
  " HIP HOP",
  "  OLDIES"
};


unsigned long lastTime;             // records millis each time we scan a tag, so we only scan at set intervals
unsigned long lastTimeButton;       // records millis each time we skip a song, to debounce button
unsigned long lastTimeVol;          // records millis each time we skip a song, keeps from constantly refresshing if hovering on edge of threshold
int scanTries = 0;                  // if scan fails, we can try a few times before stopping the music
int setVolume = 25;                 // the volume for the DFPlayer Mini (0-30)
bool isPlaying = false;

Adafruit_SSD1331 display = Adafruit_SSD1331(cs, dc, mosi, sclk, rst);   // initialize the HiLetgo SSD1331 display
DFRobotDFPlayerMini player;                                             // initialize the DFPlayer Mini
PN532_I2C pn532_i2c(Wire);                                              // initialize the RFID scanner
NfcAdapter nfc = NfcAdapter(pn532_i2c);                                 // initialize the RFID scanner (don't know why it is two separate things)

void setup(void) {
  Serial1.begin(9600);                                                  // hardware serial port for talking to DFPlayer Mini

  pinMode(BUTTON, INPUT_PULLUP);                                        // for "next track" button

  display.begin();                                                      // start display and draw a black screen
  display.fillScreen(BLACK);
  nfc.begin();                                                          // start RFID scanner

  delay(500);                                                           // wait for those things to start

  lastTime = millis();                                                  // record time
  lastTimeButton = millis();                                            // record time again
  lastTimeVol = millis();                                               // record time again, again

  if (player.begin(Serial1)) {                                          // hopefully DFPlayer starts properly
    
  } else {
    display.setCursor(22, 26);
    display.setTextColor(YELLOW);
    display.setTextSize(1);
    display.print("Failed Start");
  }
  drawLogo();                                                           // draw the "kaboom box" logo
  delay(5000);                                                          // wait for users to appreciate the logo
  display.fillScreen(BLACK);
  setVolume = analogRead(A0) / 34;
  updateVol();                                                          // set the volume and draw the volume circle
  updateIcon(99);                                                       // draw the waiting icon
  delay(500);                                                           // wait to make sure everything is peachy
}

void loop() {
  if ((millis() - lastTime) > 3000) {                                           // only lets us check a tag once per second (trying too frequently may or may not cause issues)
    lastTime = millis();
    scanTag();
  }

  if ((digitalRead(BUTTON) == LOW) && ((millis() - lastTimeButton) > 1000)) {   // only lets us skip the song once per second (dirty debounce)
    lastTimeButton = millis();
    player.next();
  }

  if ((millis() - lastTimeVol) > 50) {                                           // only lets us check a tag once per second (trying too frequently may or may not cause issues)
    lastTimeVol = millis();
    int newVolume = analogRead(A0) / 34;
    if (newVolume != setVolume) {
      setVolume = newVolume;
      updateVol();
    }
  }

}

void scanTag() {                                    // attempts to scan a tag
  if (nfc.tagPresent()) {                           // true if ANY tag scans
    NfcTag tag = nfc.read();                        // read the tag data
    tagId = tag.getUidString();                     // get tag's ID
    playTag(tagId);                                 // tries to play the scanned tag ID
    scanTries = 0;                                  // reset tries counter
  } else if (isPlaying == true) {
    if (scanTries > 0) {                            // increase tries counter (so music doesn't stop for a misread)
      player.stop();                                // stops playing all music
      isPlaying = false;
      updateVol();                                  // first clears the screen, then redraws the volume circle
      updateIcon(99);                               // waiting icon (any number greater than the tag array will show the waiting icon)
      previousId = "";                              // so we can restart the same tag, even if we just stopped it
      scanTries = 0;                                // resets tries counter
    }
    scanTries++;                                    // increments tries counter
  } else {

  }
}

void playTag(String scannedId) {                    // checks the tag after successful scan
  if (scannedId == previousId) {                    // doesn't do anything if we're already playing that tag
  } else {
    previousId = scannedId;                         // remembers the currently playing tag
    for (int i = 0; i < sizeof(tagList); i++) {     // check if tag matches any we stored in the array
      if (scannedId == tagList[i]) {
        player.stop();                             
        player.loopFolder(i+1);                     // plays the folder of matching the array entry (array index + 1, since folders start at 1 and not 0)
        isPlaying = true;
        updateIcon(i);                              // updates the playing icon
        break;                                      // exits the for loop so we don't keep checking extra tags
      }
    }
  }
}

void updateVol() {                                  // updates player volume and draws volume circle
  player.volume(setVolume);
  volDots();
}

void updateIcon(int icon){                          // draws the icon for waiting or music, with genre label defined in the array at the top
  display.fillCircle(48, 32, 22, BLACK);
  if (icon < sizeof(tagList)) {                     // checks if "icon" variable is within the array of tags
    display.drawBitmap(30,16,music, 32, 32, PINK);  // places the music note icon
    display.setCursor(22, 26);
    display.setTextColor(YELLOW);
    display.setTextSize(1);
    display.print(nameList[icon]);                  // prints the genre from the list
  } else {
    display.drawBitmap(32,16,wait, 32, 32, WHITE);  // places the waiting icon if "icon" variable is anything more than the number of array elements
  }
}

void volDots() {
  display.fillCircle(48, 4, 1, BLACK);
  display.fillCircle(64, 9, 1, BLACK);
  display.fillCircle(75, 23, 2, BLACK);
  display.fillCircle(75, 41, 2, BLACK);
  display.fillCircle(64, 55, 3, BLACK);
  display.fillCircle(48, 60, 3, BLACK);
  display.fillCircle(32, 55, 4, BLACK);
  display.fillCircle(21, 41, 4, BLACK);
  display.fillCircle(21, 23, 5, BLACK);
  display.fillCircle(32, 9, 6, BLACK);

  if (setVolume > 0) {display.fillCircle(48, 4, 1, GREEN);}
  if (setVolume > 2) {display.fillCircle(64, 9, 1, GREEN);}
  if (setVolume > 5) {display.fillCircle(75, 23, 2, GREEN);}
  if (setVolume > 8) {display.fillCircle(75, 41, 2, GREEN);}
  if (setVolume > 11) {display.fillCircle(64, 55, 3, GREEN);}
  if (setVolume > 14) {display.fillCircle(48, 60, 3, GREEN);}
  if (setVolume > 17) {display.fillCircle(32, 55, 4, GREEN);}
  if (setVolume > 20) {display.fillCircle(21, 41, 4, GREEN);}
  if (setVolume > 23) {display.fillCircle(21, 23, 5, GREEN);}
  if (setVolume > 26) {display.fillCircle(32, 9, 6, GREEN);}
}

void drawLogo() {                   // draws the "kaboom box" logo in three parts (more efficient than a full-color bitmap)
  display.drawBitmap(16,0,logo1, 64, 64, PINK);
  display.drawBitmap(16,0,logo2, 64, 64, YELLOW);
  display.drawBitmap(16,0,logo3, 64, 64, RED);
}